package dk.nsp.cbs.perf.util;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import java.util.Arrays;
import java.util.Optional;
import java.util.stream.Stream;

public class Xml {

  public static Optional<String> attr(Element elm, String name) {
    final String attr = elm.getAttribute(name);
    if (attr.isEmpty()) {
      return Optional.empty();
    }
    return Optional.of(attr);
  }

  public static Optional<String> childText(Element elm, String name) {
    return childOptional(elm, name).map(Node::getTextContent);
  }

  /**
   * The names are alternatives -- if either one of them matches, the
   * first child is returned.
   */
  public static Optional<Element> childOptional(Element elm, String... names) {
    final NodeList childNodes = elm.getChildNodes();
    for (int i = 0; i < childNodes.getLength(); i++) {
      final Node item = childNodes.item(i);
      if (item.getNodeType() != Node.ELEMENT_NODE) {
        continue;
      }
      final Element element = (Element) item;
      if (Arrays.stream(names)
                .anyMatch(name ->
                            (element.getLocalName() != null && element.getLocalName().equals(name))
                              || element.getTagName().equals(name))) {
        return Optional.of(element);
      }
    }
    return Optional.empty();
  }

  public static Stream<Element> children(Element elm) {
    final Stream.Builder<Element> builder = Stream.builder();
    final NodeList childNodes = elm.getChildNodes();
    for (int i = 0; i < childNodes.getLength(); i++) {
      final Node item = childNodes.item(i);
      if (item.getNodeType() == Node.ELEMENT_NODE) {
        builder.add((Element) item);
      }
    }
    return builder.build();
  }

  public static Stream<Element> children(Element elm, String name) {
    final Stream.Builder<Element> builder = Stream.builder();
    final NodeList childNodes = elm.getChildNodes();
    for (int i = 0; i < childNodes.getLength(); i++) {
      final Node item = childNodes.item(i);
      if (item.getNodeType() != Node.ELEMENT_NODE) {
        continue;
      }
      final Element element = (Element) item;
      if ((element.getLocalName() != null && element.getLocalName().equals(name))
        || element.getTagName().equals(name)) {
        builder.add(element);
      }
    }
    return builder.build();
  }
}
