/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2018 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.nas.sms.seal;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import javax.xml.namespace.NamespaceContext;

public class SealNamespaceContext implements NamespaceContext {

	private final static Map<String, String> nsMap = new HashMap<String, String>();

	static {
		nsMap.put("hsuid", "http://www.nsi.dk/hsuid/2012/03/hsuid-1.0.xsd");
		nsMap.put("wssec", "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd");
		nsMap.put("medcom", "http://www.medcom.dk/dgws/2006/04/dgws-1.0.xsd");
		nsMap.put("soap", "http://schemas.xmlsoap.org/soap/envelope/");
		nsMap.put("saml", "urn:oasis:names:tc:SAML:2.0:assertion");
		nsMap.put("ds", "http://www.w3.org/2000/09/xmldsig#");
	}

	@Override
	public String getNamespaceURI(String prefix) {
		return nsMap.get(prefix);
	}

	@Override
	public String getPrefix(String namespaceURI) {
		return nsMap.entrySet().stream().filter(e -> e.getValue().equals(namespaceURI)).map(Entry::getKey).findFirst().orElse(null);
	}

	@Override
	public Iterator<?> getPrefixes(String namespaceURI) {
		return nsMap.entrySet().stream().filter(e -> e.getValue().equals(namespaceURI)).map(Entry::getKey).iterator();
	}
}
