package dk.nsp.cbs.perf.runner;

import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Properties;

import dk.nsp.cbs.perf.config.CredentialVaultInfo;
import dk.nsp.cbs.perf.config.NasAccess;
import dk.nsp.cbs.perf.config.NasConfiguration;
import dk.nsp.cbs.perf.nas.DGWSService;
import dk.nsp.cbs.perf.nas.NasService;
import dk.nsp.cbs.perf.service.CbsClient;
import dk.nsp.cbs.perf.service.StsJwtClient;
import dk.nsp.cbs.perf.service.vault.VaultService;
import dk.sosi.seal.pki.Federation;
import dk.sosi.seal.pki.SOSITestFederation;
import dk.sosi.seal.vault.CredentialVault;
import org.apache.http.client.HttpClient;
import org.apache.http.impl.client.DefaultHttpClient;

public class SimpleRunner {

  public static void main(String[] args) {

    final CredentialVaultInfo idpVaultInfo = makeCredentialVaultInfo("unitteststs-nbo.keystore","Test1234");
    final CredentialVaultInfo signingVaultInfo = makeCredentialVaultInfo("TEST whitelisted SP SOSI alias.jks", "Test1234");

    final String stsJwtUrl = "http://test1-cnsp.ekstern-test.nspop.dk:8080/sts/services/JWT2Idws";
    final String stsDgwsUrl = "http://test2.ekstern-test.nspop.dk:8080/sts/services/NewSecurityTokenService";
    final String cbsUrl = "http://test2-cnsp.ekstern-test.nspop.dk:8080/cbs-service/CoronaPassWebService";

    final String stsJwtAuduence = "https://audience.nspop.dk/cbs";
    final String stsJwtKid = "ssi_foces_2021";
    final String stsJwtScope = "nsp-cbs sosi-sts";

    final NasConfiguration nasConfiguration = new NasConfiguration() {
      @Override public String topicExpressionDialect() { return "http://docs.oasis-open.org/wsn/t-1/TopicExpression/Simple"; }
      @Override public String pullpointFactoryEndpoint() { return "https://test1.ekstern-test.nspop.dk:8443/nas2/pullpointfactory"; }
      @Override public String subscriptionManagerEndpoint() { return "https://test1.ekstern-test.nspop.dk:8443/nas2/subscriptionmanager"; }
      @Override public int maxPullSize() { return 100; }
      @Override public String topic() { return "http://nspop.dk/GreenCertificate/2021/04/01:JobCompleted"; }
    };
    final NasAccess nasAccess = new NasAccess() {
      @Override public String organizationName() { return "Sundhedsdatastyrelsen"; }
      @Override public String organizatonCode() { return "30808460"; }
      @Override public String systemName() { return "NSP-COVID19-SMS-TEST"; }
    };
    final CredentialVaultInfo nasAccessVaultInfo = makeCredentialVaultInfo("FOCES_gyldig.p12", "Test1234");

    final Path vaultDir = Paths.get(".");
    final HttpClient httpClient = new DefaultHttpClient();
    final VaultService vaultService = new VaultService(vaultDir::resolve);

    final CredentialVault idpVault = vaultService.getVaultFromResource(idpVaultInfo);
    final CredentialVault signingVault = vaultService.getVaultFromResource(signingVaultInfo);
    final CredentialVault nasVault = vaultService.getVaultFromResource(nasAccessVaultInfo);

    final Federation federation = new SOSITestFederation(new Properties());

    final StsJwtClient stsJwtClient = new StsJwtClient(stsJwtUrl,
                                                       httpClient,
                                                       stsJwtAuduence,
                                                       stsJwtKid,
                                                       idpVault,
                                                       signingVault,
                                                       stsJwtScope);

    final CbsClient cbsClient = new CbsClient(cbsUrl, httpClient, federation, signingVault, System.out::println);

    final NasService nasService = new NasService(nasConfiguration,
                                                 new DGWSService(stsDgwsUrl,
                                                                 nasAccess,
                                                                 nasVault,
                                                                 httpClient,
                                                                 federation),
                                                 httpClient);

    // final Subscription subscription = nasService.subscribe(nasTopic);
    // final List<String> strings = nasService.pullJobIds(subscription);

//    final Subscription subscription = nasService.subscribe(nasConfiguration.topic());
//    System.out.println("Subscription is" + subscription.getPullpointId() + "," + subscription.getSubscriptionId());

    new Runner.TestRunner(stsJwtClient, cbsClient, "2807764072").run();
  }

  private static CredentialVaultInfo makeCredentialVaultInfo(String filename, String password) {
    return new CredentialVaultInfo() {
      @Override
      public String name() {
        return filename;
      }

      @Override
      public String password() {
        return password;
      }
    };
  }
}
