package dk.nsp.cbs.perf.runner;

import com.google.common.collect.ImmutableList;

import java.io.IOException;
import java.io.PrintStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.function.Predicate;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import java.util.stream.StreamSupport;

public class Splitter {

  private static<T> Predicate<T> not(Predicate<T> inner) {
    return t -> !inner.test(t);
  }

  private static Stream<String> shuffle(Stream<String> stream) {
    final List<String> c = stream.collect(Collectors.toList());
    Collections.shuffle(c);
    return c.stream();
  }

  public static void main(String[] args) throws IOException {
    final long cap;
    if (args.length < 2) {
      System.err.println("Please provide at least cpr.txt and hosts.txt file names as argument. Cap size is optional.");
    } else {
      final Path totalFile = Paths.get(args[0]);
      final Path hostsFile = Paths.get(args[1]);
      if (args.length > 2) {
        cap = Long.getLong(args[2]);
      } else {
        cap = 0;
      }
      if (!Files.exists(totalFile) || !Files.exists(hostsFile)) {
        System.err.printf("Both %s and %s must exist.", totalFile, hostsFile);
        System.exit(-1);
      }

      output(totalFile, hostsFile, cap);
    }
  }

  public static<T> Stream<T> take(Iterator<T> ts, long count) {
    final Iterable<T> iterable = () -> ts;
    return StreamSupport.stream(iterable.spliterator(), false).limit(count);
  }

  static void output(Path totalFile, Path hostsFile, long cap) throws IOException {
    final ImmutableList<String> hosts = Files.lines(hostsFile)
                                               .filter(not(String::isEmpty))
                                               .collect(ImmutableList.toImmutableList());

    final long nodes = hosts.size();
    final long cprCount = (cap == 0 ? Files.lines(totalFile)
                               .filter(not(String::isEmpty))
                               .count() : cap);


    final long perNodes = cprCount / nodes;
    System.out.printf("Putting %d cpr numbers into each file [hosts=%d, cpr=%d, cap=%d].\n", perNodes, nodes, cprCount, cap);

    try(final Stream<String> cprStream = shuffle(Files.lines(totalFile))
                                              .filter(not(String::isEmpty))
                                              .limit(cprCount)) {

      final Iterator<String> cprIterator = cprStream.iterator();

      hosts.stream()
           .map(host -> totalFile.toAbsolutePath().getParent().resolve(String.format("%s-cpr.txt", host)))
           .forEach(hostCprFile -> {
             try (PrintStream out = new PrintStream(Files.newOutputStream(hostCprFile))) {
               take(cprIterator, perNodes)
                 .forEach(out::println);
             } catch (IOException e) {
               throw new RuntimeException(e);
             }
           });
    }
  }
}
