package dk.nsp.cbs.perf.runner;

import dk.nsp.cbs.perf.nas.Subscription;
import org.junit.Before;
import org.junit.Test;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Random;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

/**
 * @author jns, Arosii Information Systems A/S
 */
public class FixSubscriptionTest {
  private final Path f = Paths.get("./nas-pp-sub.dat");
  private final String ppId = "http://localhost:8081/pullpoint/service/e4b99ea9-b2dd-4de3-89f5-a38b86b402fd";
  private final String subId = "http://localhost:8083/subscriptionmanager/service/ab48bed1-408b-49ff-a867-cd5253758c2c";
  private final String ppId2 = "http://localhost:8081/pullpoint/service/e4b99ea9-b2dd-4de3-7777-a38b86b402fd";
  private final String subId2 = "http://localhost:8083/subscriptionmanager/service/ab48bed1-408b-49ff-8888-cd5253758c2c";

  @Before
  public void fixFile() throws IOException {
    if (Files.exists(f)) {
      Files.delete(f);
    }
  }

  @Test
  public void testWriteSub() {
    Subscription s = new Subscription(
        new Random().nextLong(),
        "testWriteSub",
        ppId,
        subId);
    try {
      TestDriver.SubDat d = TestDriver.fixSubscription(s);
      assertTrue("file nas-pp-sub.dat does not exist", Files.exists(f));
    } catch (IOException e) {
      fail();
    }
  }

  @Test
  public void testContents() throws IOException {
    Subscription s = new Subscription(
        new Random().nextLong(),
        "blah",
        ppId,
        subId);
    TestDriver.fixSubscription(s);
    String[] dat = new String(Files.readAllBytes(f)).split(",");
    assertEquals("ppId must match", ppId, dat[0]);
    assertEquals("subId must match", subId, dat[1]);
  }

  @Test
  public void testNoPrior() throws IOException {
    Subscription s = new Subscription(
        new Random().nextLong(),
        "blah",
        ppId,
        subId);
    TestDriver.SubDat d = TestDriver.fixSubscription(s);
    assertNull("SubDat should be null", d);
  }

  @Test
  public void testWithPrior() throws IOException {
    Subscription s = new Subscription(
        new Random().nextLong(),
        "blah",
        ppId,
        subId);

    TestDriver.fixSubscription(s);

    Subscription s2 = new Subscription(
        new Random().nextLong(),
        "blah",
        ppId2,
        subId2);

    TestDriver.SubDat d = TestDriver.fixSubscription(s2);
    assertEquals("ppId must match", ppId, d.getPpId());
    assertEquals("subId must match", subId, d.getSubId());
  }
}