package dk.nsp.cbs.perf.runner;

import com.google.common.collect.ImmutableList;
import com.google.common.jimfs.Configuration;
import com.google.common.jimfs.Jimfs;
import org.junit.Assert;
import org.junit.Test;

import java.io.IOException;
import java.nio.file.FileSystem;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import java.util.stream.Stream;

public class SplitterTest {

  private final FileSystem fs = Jimfs.newFileSystem(Configuration.forCurrentPlatform());

  private static Stream<String> lines(Path path) {
    try {
      return Files.lines(path);
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }

  @Test
  public void testSplitter() throws IOException {
    final Path root = fs.getPath("/splittertest");

    Files.createDirectories(root);

    final Path cprFile = root.resolve("cpr.txt");
    Files.write(cprFile, IntStream.range(0, 35)
                                  .map(i -> i + 1111111118)
                                  .mapToObj(Integer::toString)
                                  .collect(Collectors.toList()));
    final Path hostFile = root.resolve("hosts.txt");
    final ImmutableList<String> hosts = ImmutableList.of("aHost",
                                                      "bHost",
                                                      "cHost");
    Files.write(hostFile, hosts);

    Splitter.output(cprFile, hostFile, 0);

    Assert.assertTrue(hosts.stream()
                           .map(host -> root.resolve(String.format("%s-cpr.txt", host)))
                           .allMatch(Files::exists));

    Assert.assertTrue(
      hosts.stream()
           .map(host -> root.resolve(String.format("%s-cpr.txt", host)))
           .map(SplitterTest::lines)
           .map(Stream::count)
           .peek(System.out::println)
           .allMatch(cprs -> cprs >= 10));
  }

  @Test
  public void testSplitterCap() throws IOException {
    final Path root = fs.getPath("/splittertest");

    Files.createDirectories(root);

    final Path cprFile = root.resolve("cpr.txt");
    Files.write(cprFile, IntStream.range(0, 35)
        .map(i -> i + 1111111118)
        .mapToObj(Integer::toString)
        .collect(Collectors.toList()));
    final Path hostFile = root.resolve("hosts.txt");
    final ImmutableList<String> hosts = ImmutableList.of("aHost",
        "bHost",
        "cHost");
    Files.write(hostFile, hosts);

    Splitter.output(cprFile, hostFile, 20);

    Assert.assertTrue(hosts.stream()
        .map(host -> root.resolve(String.format("%s-cpr.txt", host)))
        .allMatch(Files::exists));

    Assert.assertTrue(
        hosts.stream()
            .map(host -> root.resolve(String.format("%s-cpr.txt", host)))
            .map(SplitterTest::lines)
            .map(Stream::count)
            .peek(System.out::println)
            .allMatch(cprs -> cprs < 7));
  }
}