package dk.nsp.cbs.perf.service.vault;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Properties;

import dk.nsp.cbs.perf.config.CredentialVaultInfo;
import dk.sosi.seal.model.SignatureUtil;
import dk.sosi.seal.vault.CredentialVault;
import dk.sosi.seal.vault.FileBasedCredentialVault;
import dk.sosi.seal.vault.GenericCredentialVault;

public class VaultService {

  private final FileResolver resolver;

  public VaultService(FileResolver resolver) {
    this.resolver = resolver;
  }

  private static GenericCredentialVault getEmptyVault(String password) {
    return new GenericCredentialVault(SignatureUtil.setupCryptoProviderForJVM(), password);
  }

  public CredentialVault getVaultFromResource(CredentialVaultInfo credentialVaultInfo) {
    final Path credVaultPath = resolver.resolve(credentialVaultInfo.name());
    try {
      return getVaultFromResource(credVaultPath, credentialVaultInfo.password());
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }

  private CredentialVault getVaultFromResource(Path path, String password) throws IOException {
    if (path.toString().endsWith("pkcs12") || path.toString().endsWith("p12")) {
      final GenericCredentialVault vault = getEmptyVault(password);
      byte[] pkcs12 = Files.readAllBytes(path);
      vault.setSystemCredentialPair(new ByteArrayInputStream(pkcs12), password);
      return vault;
    } else {
      final Properties properties = SignatureUtil.setupCryptoProviderForJVM();
      return new FileBasedCredentialVault(properties, path.toFile(), password);
    }
  }
}
