package dk.nsp.cbs.perf.stub;

import java.util.Optional;
import java.util.Properties;

import dk.nsp.cbs.perf.service.CbsClient;
import dk.nsp.cbs.perf.util.Xml;
import dk.sosi.seal.xml.XmlUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class CbsProtocol {

  private Logger logger = LoggerFactory.getLogger(getClass());

  private final CbsServer cbsServer;

  public CbsProtocol(CbsServer cbsServer) {
    this.cbsServer = cbsServer;
  }

  private static final String testString = "<?xml version=\"1.0\"?>\n" +
    "<soap:Envelope xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\">\n" +
    "  <soap:Body>\n" +
    "    <GetCoronaPassResponse xmlns=\"http://sundhedsdatastyrelsen.dk/coronapass/2021/03/coronapass-web-service\">\n" +
    "      <CoronaPasses>\n" +
    "        <Pass>\n" +
    "          <Type>EU</Type>\n" +
    "          <JWT>6BFC$B5Z7+J2M433FH*:R71FZ7F+IMVC3L6Q6WCPCGU+CLGE86U9.PK0E5QV2UF$XCL66N93/.DMNK4FO/34*LGTP2K+Q/RO602W5P%H9H%RM5A8I1$CAROQC1252AQIS*ZUB17+K74VMWZS8/E8V2CY2M2HO8J:0H1XI$E5KCOCC07NB9EB$KC0:2W-R$*2RYRQ6PW.VYKVUNC4QK4*U7+QZ3KFTOK56LSLQV4JNB0ML7RINLGBPK7U7T.4PNI%CBEY3ECF7W37XK.73JA9IQGKOKK62 9VPG0*CIFGBLK5P*I$DNCE4PN8LFNO769AHBILWRSPA644AYKUOOL$MRIFCUR0+.2%9J$/2VHS1ML325 80E:2Z:C1 TI8PP$P84E3-3CNDD2C/4ED D.3NL9BW7P.BQQZFE8QI 1Q9G/ECHW8NS4VN6.%7JMN:Y8JLUY9DEF90319JF4A8.TS+4NSDMHD1:-JW 02DA5*P4$OWMAHLCJ2Q266JA0TNBC0QWOBO$TO8QM%A5P7QVPN3HUBAHJF0KA EUPH2N-JEM753B4KRS3CU.7FYNBUCVND*9E7VJS:VZBQH2GWTDV2JQHGZ7O6:36F6+$RU/UEUR76IY17XZN706YCFS FMCSO04R38C4OA$VI0C470/J9H1</JWT>\n" +
    "          <Properties>\n" +
    "            <Key>eu:keytype</Key>\n" +
    "            <Value>digital_green_certificate</Value>\n" +
    "          </Properties>\n" +
    "        </Pass>\n" +
    "        <Pass>\n" +
    "          <Type>DK</Type>\n" +
    "          <JWT>eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJleHAiOjE2MTc3MTg3MTYsImlhdCI6MTYxNjQwMzY1NywianRpIjoiOWQwMjhmMjktMjU0Ni00ZDU5LThjN2EtOGJjOWI1MmEzNmU4In0.c-FZaQWVoosnJA18uoOItgX10v9XHDHkAyNgfDgdipQ</JWT>\n" +
    "        </Pass>\n" +
    "        <Pass>\n" +
    "          <Type>DKPLUS</Type>\n" +
    "          <JWT>eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJleHAiOjE2MTc3MTg4MTEsImlhdCI6MTYxNjQwMzY1NywianRpIjoiOWQwMjhmMjktMjU0Ni00ZDU5LThjN2EtOGJjOWI1MmEzNmU4IiwibGVnYWxOYW1lIjoiSmFudXMgTmllbHNlbiIsImRhdGVPZkJpcnRoIjoiMTk4NS0xMi0wMyJ9.YQpCr3igO61wBjcrSIsIXGp_5FBTV0KQUTw-ZrZDEOg</JWT>\n" +
    "        </Pass>\n" +
    "      </CoronaPasses>\n" +
    "    </GetCoronaPassResponse>\n" +
    "  </soap:Body>\n" +
    "</soap:Envelope>";

  private final String testMissString = "<?xml version=\"1.0\"?>\n" +
    "<soap:Envelope xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\">\n" +
    "  <soap:Body>\n" +
    "    <GetCoronaPassResponse xmlns=\"http://sundhedsdatastyrelsen.dk/coronapass/2021/03/coronapass-web-service\">\n" +
    "      <FutureResult>" +
    "        <JobId>%s</JobId>" +
    "      </FutureResult>" +
    "    </GetCoronaPassResponse>\n" +
    "  </soap:Body>\n" +
    "</soap:Envelope>";

  public String call(String body) {
    final Document document = XmlUtil.readXml(new Properties(),
                                              body,
                                              false);
    final Element root = document.getDocumentElement();

    final Optional<String> identifier = Xml.children(root, "Body")
                                          .findFirst()
                                          .flatMap(soapBody -> Xml.children(soapBody, "GetCoronaPassRequest").findFirst())
                                          .flatMap(req -> Xml.childText(req, "Identifier"));
    logger.info("CBS called with "+identifier.get());

    final CbsClient.CbsResponse response = cbsServer.call(identifier.get());
    if (!response.isHit()) {
      return String.format(testMissString, response.getJobId());
    }
    return testString;
  }

}
