package dk.nsp.cbs.perf.stub;

import java.time.Instant;
import java.util.Comparator;
import java.util.PriorityQueue;
import java.util.Random;

import com.google.common.collect.ImmutableList;

/* this class also mock the data collection processes (kafka etc.) */
public class NasServer {

  public static class Job {
    public final Instant finished;
    public final String jobId;
    public final String cpr;

    public Job(Instant finished, String jobId, String cpr) {

      this.finished = finished;
      this.jobId = jobId;
      this.cpr = cpr;
    }

    public Instant getFinished() {
      return finished;
    }
  }

  private final DataCache dataCache;

  public NasServer(DataCache dataCache) {
    this.dataCache = dataCache;
  }

  private final PriorityQueue<Job> jobs = new PriorityQueue<>(100,
                                                              Comparator.comparing(Job::getFinished));

  public String createPullPoint() {
    return "http://localhost:8080/nas2/pullpoint/pp42";
  }

  public String createSubscription() { return "http://localhost:8080/nas2/subscriptionmanager/sub42"; }

  // bla bla bla thread-safe, if I say so
  public synchronized Iterable<Job> getJobIds() {
    final Instant now = Instant.now();
    final ImmutableList.Builder<Job> res = new ImmutableList.Builder<>();

    while (true) {
      final Job peek = jobs.peek();
      if (peek == null) {
        break;
      }
      if (peek.finished.isBefore(now)) {
        res.add(jobs.poll());
      } else {
        break;
      }
    }
    return res.build();
  }

  private final Random random = new Random();

  public synchronized void push(String jobId, String cpr) {
    final Instant instant = Instant.now().plusSeconds(random.nextInt(8));
    jobs.add(new Job(instant, jobId, cpr));
    dataCache.put(instant, cpr);
  }
}
