package dk.nsp.cbs.perf.stub;

import java.util.function.Function;

import spark.Route;
import spark.Spark;

public class StubStarter {

  private static Route wrap(Function<String, String> strFn) {
    return (req, res) -> {
      try {
        res.header("Content-Type", "application/xml");
        return strFn.apply(req.body());
      } catch (Exception e) {
        e.printStackTrace();
        return e.getMessage();
      }
    };
  }

  public static void main(String[] args) {

    final DataCache dataCache = new DataCache();
    final NasServer nasServer = new NasServer(dataCache);
    final JobPlacementService jobPlacementService = new JobPlacementService(nasServer);
    final CbsServer cbsServer = new CbsServer(dataCache, jobPlacementService);

    final NasProtocol nasProtocol = new NasProtocol(nasServer);
    final CbsProtocol cbsProtocol = new CbsProtocol(cbsServer);

    // https://zetcode.com/java/undertow/
    Spark.port(8080);
    Spark.post("/nas2/pullpointfactory", wrap(nasProtocol::pullPointFactory));
    Spark.post("/nas2/pullpoint/*", wrap(nasProtocol::pullpoint));
    Spark.post("/nas2/subscriptionmanager", wrap(nasProtocol::subscriptionManager));
    Spark.post("/nas2/subscriptionmanager/*", wrap(nasProtocol::subscriptionManager));
    Spark.post("/cbs-service/CoronaPassWebService", wrap(cbsProtocol::call));
                
  }


}
